import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {TieToActivityModalComponent} from './tie-to-activity-modal.component';
import {of} from 'rxjs';
import {FormsModule} from '@angular/forms';
import {DirtyGuard} from '../../confirm-navigation-if-dirty.framework';
import {By} from '@angular/platform-browser';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {CarePlanService} from '../../api/care-plan.service';
import {Activity} from '@nspop/gm-web-facade-api';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {TrimToLengthPipe} from '../../pipes/trim-to-length.pipe';
import {ModalComponent} from '../../modals/modal/modal.component';
import {ActivityStatusPipe} from './activity-status.pipe';
import { SimpleChange } from '@angular/core';

describe('TieToActivityModalComponent', () => {
  let component: TieToActivityModalComponent;
  let fixture: ComponentFixture<TieToActivityModalComponent>;
  let carePlanService: CarePlanService;

  beforeEach(async () => {
    carePlanService = {
      activities$: of([]),
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ TieToActivityModalComponent, TrimToLengthPipe, ModalComponent, ActivityStatusPipe],
      imports: [FormsModule, AngularMaterialModule, NoopAnimationsModule],
      providers: [
        {provide: CarePlanService, useValue: carePlanService},
        {provide: DirtyGuard, useValue: {canDeactivate: () => true}}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(TieToActivityModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('creates a row for each activity', fakeAsync(() => {
    carePlanService.activities$ = of([
      {title: 'someTitle', id: '1', gestationWeek: '2+2'},
      {title: 'some other Title', id: '2', gestationWeek: '5+1'},
      {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'},
      {title: 'the otherest title', id: '4', gestationWeek: '22+6'},
    ] as any);
    component.ngOnInit();
    tick();
    fixture.detectChanges();
    expect(component.selectableActivities.length).toBe(4);
  }));

  it('should not show activities with observations when isObsActivitiesShown is false', fakeAsync(() => {
    carePlanService.activities$ = of([
      {title: 'show me', id: '1', gestationWeek: '2+2'},
      {title: 'show me too', id: '2', gestationWeek: '5+1'},
      {title: 'dont show me', id: '3', gestationWeek: '9+2', observations: [{id: 'abcd'}]},
      {title: 'but show me', id: '4', gestationWeek: '22+6'},
    ] as any);
    component.ngOnInit();
    tick();
    fixture.detectChanges();
    expect(component.selectableActivities.length).toBe(4);

    component.isObsActivitiesShown = false;
    component.ngOnChanges({
      isObsActivitiesShown: new SimpleChange(null, false, true)
    });
    fixture.detectChanges();
    expect(component.selectableActivities.length).toBe(3);
  }));

  it('updates ngModel when user clicks "select"', fakeAsync(() => {
    carePlanService.activities$ = of([
      {title: 'someTitle', id: '1', gestationWeek: '2+2'},
      {title: 'some other Title', id: '2', gestationWeek: '5+1'},
      {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'},
      {title: 'the otherest title', id: '4', gestationWeek: '22+6'},
    ] as any);
    const activity = {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'} as any;
    component.ngOnInit();
    tick();
    component.innerModel = activity;
    fixture.detectChanges();
    expect(component.currentValue).toBeFalsy();
    component.pickActivity();
    expect(component.currentValue).toEqual(activity);
    expect(component.displayText).toEqual('someeven mroe other Title');
  }));

  it('leaves ngModel unchanged when user clicks "cancel"', fakeAsync(() => {
    carePlanService.activities$ = of([
      {title: 'someTitle', id: '1', gestationWeek: '2+2'},
      {title: 'some other Title', id: '2', gestationWeek: '5+1'},
      {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'},
      {title: 'the otherest title', id: '4', gestationWeek: '22+6'},
    ] as any);
    component.ngOnInit();
    tick();
    component.innerModel = {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'} as any;
    fixture.detectChanges();
    expect(component.currentValue).toBeFalsy();
    component.handleClose();
    expect(component.currentValue).toBeFalsy();
    expect(component.displayText).toEqual('');
  }));

  it('sets display text if activity is provided on init', fakeAsync(() => {
    carePlanService.activities$ = of([
      {title: 'someTitle', id: '1', gestationWeek: '2+2'},
      {title: 'some other Title', id: '2', gestationWeek: '5+1'},
      {title: 'someeven mroe other Title', id: '3', gestationWeek: '9+2'},
      {title: 'the otherest title', id: '4', gestationWeek: '22+6'},
    ] as any);
    const activity = {title: 'someeven mroe other Title', id: '3'} as Activity;
    component.writeValue(activity);
    component.ngOnInit();
    fixture.detectChanges();
    tick();
    expect(component.currentValue).toEqual(activity);
    expect(component.displayText).toEqual('someeven mroe other Title');
  }));

  it('opens activity selector panel when clicked', fakeAsync(() => {
    expect(component.modalComponent?.isModalVisible).toBeFalse();
    const widget = fixture.debugElement.query(By.css('.form-field-wrapper')).nativeElement;
    widget.dispatchEvent(new Event('click'));
    expect(component.modalComponent?.isModalVisible).toBeTrue();
  }));

  it('cant open activity-selector when disabled', fakeAsync(() => {
    component.setDisabledState(true);
    fixture.detectChanges();
    expect(component.modalComponent?.isModalVisible).toBeFalse();
    const widget = fixture.debugElement.query(By.css('.form-field-wrapper')).nativeElement;
    widget.dispatchEvent(new Event('click'));
    expect(component.modalComponent?.isModalVisible).toBeFalse();
  }));
});
