import { Component, forwardRef, Input, OnChanges, OnInit, SimpleChanges, ViewChild } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { ModalComponent } from '../../modals/modal/modal.component';
import { Activity } from '@nspop/gm-web-facade-api';
import { CarePlanService } from '../../api/care-plan.service';
import ActivityUtil from '../../../utils/activity.util';

export const TIE_TO_ACTIVITY_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => TieToActivityModalComponent),
  multi: true
};

@Component({
  selector: 'app-tie-to-activity-modal',
  templateUrl: './tie-to-activity-modal.component.html',
  styleUrls: [ './tie-to-activity-modal.component.scss' ],
  providers: [ TIE_TO_ACTIVITY_CONTROL_VALUE_ACCESSOR ]
})
export class TieToActivityModalComponent implements OnInit, OnChanges, ControlValueAccessor {
  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @Input() public isObsActivitiesShown = true;

  public innerModel?: Activity = undefined;
  public currentValue: Activity | undefined;
  public displayText = '';
  public disabled = false;
  public hover = false;

  public allActivities: Activity[] = [];
  public selectableActivities: Activity[] = [];

  get value(): Activity | undefined {
    return this.innerModel;
  }

  set value(value: Activity | undefined) {
    if (value?.id !== this.innerModel?.id) {
      this.innerModel = this.selectableActivities.find(x => x.id === this.currentValue?.id);
      this.onChangeCallback(value);
    }
  }

  constructor(private carePlanService: CarePlanService) { }

  ngOnInit(): void {
    this.carePlanService.activities$.subscribe(activities => {
      this.allActivities = activities;
      this.updateSelectableActivities();
      this.activityChanged();
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    const { isObsActivitiesShown } = changes;
    if (isObsActivitiesShown != null) {
      this.updateSelectableActivities();
      this.activityChanged();
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: Activity): void {
    this.currentValue = newValue;
    this.value = newValue;
    this.displayText = this.getDisplayText();
  }

  handleClose(): void {
    this.modalComponent?.setVisible(false);
    this.value = this.currentValue;
  }

  open(): void {
    if (!this.disabled) {
      this.modalComponent?.setVisible(true);
    }
  }

  pickActivity(): void {
    this.currentValue = this.innerModel;
    this.displayText = this.getDisplayText();
    this.onChangeCallback(this.innerModel);
    this.handleClose();
  }

  private onChangeCallback: (newValue?: Activity) => any = () => {};

  private onTouchedCallback: () => any = () => {};

  private updateSelectableActivities(): void {
    this.selectableActivities = this.allActivities.filter(activity => {
      if (this.isObsActivitiesShown) {
        return activity.status !== 'cancelled';
      } else {
        return activity.status !== 'cancelled' && !ActivityUtil.hasMeasurements(activity);
      }
    });
  }

  private activityChanged(): void {
    this.innerModel = this.selectableActivities.find(x => x.id === this.currentValue?.id);
    this.displayText = this.getDisplayText();
  }

  private getDisplayText(): string {
    return this.selectableActivities.find(x => x.id === this.currentValue?.id)?.title || '';
  }

}
