import {TestBed} from '@angular/core/testing';

import {AppError, CprDoesNotExistError, ErrorService, InvalidSessionError, PartialDataCardError} from './error.service';
import {PatientContextService} from './patient-context.service';
import {Router} from '@angular/router';
import {HttpClientTestingModule} from '@angular/common/http/testing';
import {appUrls} from '../../utils/URL_STRING_LITERALS';
import {HttpErrorResponse} from '@angular/common/http';

describe('ErrorService', () => {
  let service: ErrorService;
  let patientContextServiceStub: PatientContextService;
  let routerStub: Router;
  let error: any;

  beforeEach(async () => {
    patientContextServiceStub = {
      setPatientExists: () => {},
      setHasPrivatelyMarkedData: () => {}
    } as any;

    routerStub = {
      navigate: () => {
      }
    } as any;

    spyOn(console, 'error'); // Avoid logging the errors in console during test

    await TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: PatientContextService, useValue: patientContextServiceStub},
        {provide: Router, useValue: routerStub},
      ]
    }).compileComponents();
    service = TestBed.inject(ErrorService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('when an AppError is thrown', () => {
    beforeEach(() => {
      error = new AppError();
    });

    it('navigates to somethingWentWrong url', () => {
      spyOn(routerStub, 'navigate');
      service.handleError(error);
      expect(routerStub.navigate).toHaveBeenCalledTimes(1);
      expect(routerStub.navigate).toHaveBeenCalledWith([appUrls.somethingWentWrong], {
        queryParams: {
          errorId: undefined,
          errorMessage: 'Der skete en ukendt fejl'
        }
      });
    });
  });

  describe('when a CprDoesNotExistError is thrown', () => {
    beforeEach(() => {
      error = new CprDoesNotExistError();
    });

    it('sets patientContext.patientExists to false', () => {
      spyOn(patientContextServiceStub, 'setPatientExists');
      service.handleError(error);
      expect(patientContextServiceStub.setPatientExists).toHaveBeenCalledTimes(1);
      expect(patientContextServiceStub.setPatientExists).toHaveBeenCalledWith(false);
    });
  });

  it('logs any error to console.error', () => {
    error = new Error();
    service.handleError(error);
    expect(console.error).toHaveBeenCalledTimes(1);
    expect(console.error).toHaveBeenCalledWith('', error);
  });

  describe('Generic error handling', () => {
    it('should handle partialDataCardError', () => {
      spyOn(patientContextServiceStub, 'setHasPrivatelyMarkedData').and.callThrough();
      error = new PartialDataCardError();
      service.handleError(error);
      expect(patientContextServiceStub.setHasPrivatelyMarkedData).toHaveBeenCalledWith(true);
    });

    it('should handle invalid session errors', () => {
      spyOn(routerStub, 'navigate');
      error = new InvalidSessionError();
      service.handleError(error);
      expect(routerStub.navigate).toHaveBeenCalledWith([appUrls.somethingWentWrong], {
        queryParams: {
          errorId: undefined,
          errorMessage: 'Sessionen i denne fane er ikke længere gyldig. Luk venligst denne fane.'
        }
      });
    });
  });
  describe('HttpResponse tests', () => {
    it('should handle HttpErrorResponses', () => {
      error = new Error();
      error.type = '/graviditetsmappen/api/problems/not-found.html';
      const errorResponse = new HttpErrorResponse({
        error,
        status: 401,
        statusText: 'Not Allowed',
        url: '/graviditetsmappen',
        headers: {} as any});
      service.handleHttpErrorResponse(errorResponse);
    });
    it('should handle redirect requests', () => {
      error = new Error();
      error.text = '<html><head><meta http-equiv="refresh" content= url="hello"';
      const errorResponse = new HttpErrorResponse({
        error,
        status: 200,
        statusText: 'Not Allowed',
        url: '/graviditetsmappen',
        headers: {} as any});

      service.redirectIfRequested(errorResponse);
    });
  });
});
