import {ErrorHandler, Injectable, NgZone} from '@angular/core';
import {Router} from '@angular/router';
import {PatientContextService} from './patient-context.service';
import {HttpErrorResponse} from '@angular/common/http';
import {appUrls} from '../../utils/URL_STRING_LITERALS';
import {Observable, throwError} from 'rxjs';
import {CookieService} from 'ngx-cookie-service';

@Injectable({
  providedIn: 'root'
})
export class ErrorService implements ErrorHandler {

  constructor(
    private zone: NgZone,
    private router: Router,
    private patientContextService: PatientContextService,
    private cookieService: CookieService
  ) { }

  /**
   * Handles app-specific errors
   * @param error any errors thrown
   */
  handleError(error: any): void {
    let prefix: string = '';

    if (error instanceof CprDoesNotExistError) {
      this.handleCprDoesNotExistError();
      prefix = 'CprDoesNotExistError';
    }
    else if (error instanceof PartialDataCardError) {
      this.handlePartialDataCardError();
      prefix = 'PartialDataCardError';
    }
    else if (error instanceof InvalidSessionError) {
      this.handleAppError(error);
      prefix = 'InvalidSessionError';
    }
    else if (error instanceof AppError) {
      this.handleAppError(error);
      prefix = 'AppError';
    }

    if (error?.message?.includes('CprDoesNotExistError')) {
      return;
    } else if (error?.message?.includes('AppError')) {
      return;
    }
    console.error(prefix, error);
  }

  public handleHttpErrorResponse(errorResponse: HttpErrorResponse): Observable<never> {
    if (errorResponse?.error?.type === '/graviditetsmappen/api/problems/not-found.html') {
      return this.throwError(new CprDoesNotExistError(errorResponse));
    }
    return throwError(new AppError(undefined, errorResponse));
  }

  private handleCprDoesNotExistError(): void {
    this.patientContextService.setPatientExists(false);
    this.router.navigate([appUrls.cprDoesNotExist]);
  }

  private handlePartialDataCardError(): void {
    this.patientContextService.setHasPrivatelyMarkedData(true);
    this.router.navigate([appUrls.privatelyMarked]);
  }

  private handleAppError(error: AppError): void {
    this.zone.run(() => this.router.navigate([appUrls.somethingWentWrong],
      {queryParams: {errorId: error.errorId, errorMessage: error.errorMessage}}));
  }

  private throwError(error: any): Observable<never> {
    this.handleError(error);
    return throwError(error);
  }

  /**
   * Browser intercepts redirects in xhr calls and returns 200 OK
   * this will check if any login redirects were requested by SAML
   * @param errorResponse the httpErrorResponse
   */
  redirectIfRequested(errorResponse: HttpErrorResponse): void {
    const redirectHtmlStart = '<html><head><meta http-equiv="refresh" content=';
    // Regex for oiosaml.fragment Cookie: url=(.*?)\".*document\.cookie = '(.*)';

    if (errorResponse.status === 200 && errorResponse.error.text?.includes(redirectHtmlStart)) {
      const redirectUrlRegex = /url=(.*?)"/;
      const matches = redirectUrlRegex.exec(errorResponse.error.text);
      if (matches !== null && matches.length > 1) {
        this.router.navigate([appUrls.base]);
      }
    }
  }
}

// Custom App Errors
/**
 * The Generic Application error. Parent to all other errors.
 */
export interface GenericAppError {
  errorMessage: string;
  errorId?: string;
  innerError?: Error;
}

/**
 * Top level error
 */
export class AppError implements GenericAppError {
  errorId?: string;
  detailed?: boolean;

  constructor(public errorMessage: string = 'Der skete en ukendt fejl', public innerError?: Error) {
    if (!!innerError && innerError instanceof HttpErrorResponse) {
      this.errorId = innerError.error?.debugging;
      this.detailed = !!innerError.error?.detail;
      this.errorMessage = this.detailed ? innerError.error?.detail : this.errorMessage;
    }
  }
}

/**
 * Expected error for when sessionStorage does not match JSESSION
 */
export class InvalidSessionError extends AppError {
  constructor(innerError?: Error) {
    super('Sessionen i denne fane er ikke længere gyldig. Luk venligst denne fane.', innerError);
  }
}

/**
 * Expected error for when patient does not exist
 */
export class CprDoesNotExistError extends AppError {
  constructor(innerError?: Error) {
    super('CPR findes ikke', innerError);
  }
}

/**
 * Expected error for when a patient has privately marked data
 */
export class PartialDataCardError extends AppError {
  constructor(innerError?: Error) {
    super('Patienten har privatmarkeret data', innerError);
  }
}


